from flask import Flask, request, jsonify
from flask_httpauth import HTTPBasicAuth
from werkzeug.security import generate_password_hash, check_password_hash
import json
import sqlite3
import os
DB_PATH = os.path.join(os.path.dirname(__file__), 'links.db')

app = Flask(__name__)
auth = HTTPBasicAuth('CustomBasic')

users = {
    "admin": generate_password_hash("test"),
}

def regen_JSON():
    """Gets links from DB and outputs them in JSON"""
    con = sqlite3.connect(DB_PATH)
    con.row_factory = sqlite3.Row
    cur = con.cursor()
    cur.execute('SELECT url, name FROM links ORDER BY position')

    links_list = []
    for row in cur.fetchall():
      d = dict(zip(row.keys(), row))
      links_list.append(d)
    links_json = json.dumps(links_list, indent=4)

    con.close()
    return links_json

@auth.verify_password
def verify_password(username, password):
    if username in users and \
            check_password_hash(users.get(username), password):
        return username

@app.route('/editor/links', methods = ['POST']) 
@auth.login_required
def update_links():
    con = sqlite3.connect(DB_PATH)
    cur = con.cursor()
    try:
        cur.execute("begin")
        cur.execute('DELETE FROM links')
        
        links = []
        data = request.json['links']
        items = 'url', 'name', 'clicks', 'position'
        for i in range(len(data)):
            if not(all(e in data[i] for e in items)): 
                return "Bad request, some items missing from link object", 400

            url = data[i]['url']
            name = data[i]['name']
            clicks = data[i]['clicks']
            position = data[i]['position'] # TODO

            newlink = (url, name, clicks, position)
            links.append(newlink)

        cur.executemany('INSERT INTO links VALUES (?,?,?,?)', links)
        con.commit()
        data = regen_JSON()
        # TODO: Trigger a rebuild of the frontend
        outfile = open('data.json', 'w')
        print(data, file=outfile)
        outfile.close()

    except:
        cur.execute("rollback")
    return "done updating links"

@app.route('/editor/links', methods = ['GET']) 
@auth.login_required
def get_links():
    # endpoint lists all URLs and clicks, returns json object for editor.
    con = sqlite3.connect(DB_PATH)
    cur = con.cursor()
    cur.execute('SELECT position, name, url, clicks FROM links ORDER BY position')

    links_list = []
    for row in cur.fetchall():
        d = dict(zip(["position", "name", "url", "clicks"], [row[0], row[1], row[2], row[3]]))
        links_list.append(d)

    con.close()
    return jsonify(links_list)

@app.route('/clicks', methods=['POST'])
def update_clicks():
    if ('url' not in request.json or 'name' not in request.json):
        return 'url and/or name not found', 500
    else:
        url_id = request.json['url']
        url_name = request.json['name']
        con = sqlite3.connect(DB_PATH)
        cur = con.cursor()
        cur.execute("UPDATE links SET clicks=clicks + 1 WHERE url=? AND name=?", [url_id, url_name])
        con.commit()
        con.close()
        return 'ok'

if __name__ == "__main__":
    app.run(debug=True)
